# This software is licensed under the GPL, Version 2, June 1991
#
# Copyright (c) 2002 Peter Jentsch <lemuren@web.de>
#
# see LICENSE for details
#

# python imports
import re
import os
from urllib import quote as url_quote, unquote as url_unquote
from DocumentTemplate.DT_Util import html_quote
from time import time, localtime, strftime, gmtime
from string import strip,split
from types import *
from StringIO import StringIO
from os.path import normpath
from glob import glob

# Zope imports
import Globals
from Globals import DTMLFile
from DocumentTemplate.DT_Util import html_quote
from Products.PythonScripts.PythonScript import manage_addPythonScript, PythonScript
from Products.PageTemplates.ZopePageTemplate import manage_addPageTemplate, PageTemplate
from Products.PageTemplates.PageTemplateFile import PageTemplateFile

# product imports
# from Const import *

tagRegex = re.compile("<.*?>")
# where we exist on the file system
file_path = Globals.package_home(globals())
PageTemplateFileTypes = ['pt', 'zpt']
PythonScriptFileTypes = ['py', 'pys']
DTMLFileTypes = ['dtml']
method_extensions = PageTemplateFileTypes + PythonScriptFileTypes + DTMLFileTypes
image_extensions = ['gif', 'png', 'jpg', 'jpeg']

def intersect(l1, l2):
    # return intersection of two sequences
    return filter(lambda x, l=l2: x in l, l1)

def difference(l1, l2):
    # return difference of two sequences (l1 - l2)
    return filter(lambda x, l=l2: x not in l, l1)

def remove_duplicates(sequence):
    l = list(sequence)
    l.sort()
    l = filter(None, map(lambda x, y: x != y and x,l, l[1:]))
    if TupleType is type(sequence):
        return tuple(l)
    else:
        return l

def getMatchingSourceFile(base_filename, pref_types=method_extensions):
    """ in: filename:string, pref_types:list of strings w/ preferred ext.
        out: tuple(complete filename, methodtype)
        (methodtype is one of PageTemplate, DTMLMethod, PythonScript) """
    if base_filename.startswith(file_path):
        base_filename = base_filename[len(file_path):]
    if pref_types is None:
        file_type = method_extensions
    else:
        file_type = pref_types
    if ListType is type(file_type):
        for extension in file_type:
            filename = normpath(file_path + '/' + base_filename + '.' + extension)
            if os.path.exists(filename):
                file_type = extension
                break
    else:
        filename = normpath(file_path + '/' + base_filename + '.' + file_type)
    return filename, file_type

def getTemplateFile(base_filename, globals=globals()):
    filename, file_type = getMatchingSourceFile(base_filename, PageTemplateFileTypes + DTMLFileTypes)
    if file_type in PageTemplateFileTypes:
        return PageTemplateFile(base_filename, globals)
    elif file_type in DTMLFileTypes:
        return DTMLFile(base_filename, globals)
    else:
        assert 0

def getFileIds(pattern):
    ids = []
    dict = {}
    filenames = glob(os.path.normpath(pattern))
    ids = map(os.path.basename, filenames)
    ids = map(os.path.splitext, ids)
    ids = map(lambda x: x[0], ids)
    ids = filter(None, ids)
    return ids

def addPythonScript(obj,id,filename):
    f=open(filename)
    file=f.read()
    f.close()
    manage_addPythonScript(obj,id)
    obj._getOb(id).write(file)
    return getattr(obj,id)

def addDTML(obj,id,title,filename):
    f=open(filename)
    file=f.read()
    f.close()
    obj.manage_addDTMLMethod(id,title,file)
    return getattr(obj,id)

def addPageTemplate(obj,id,title,filename):
    f=open(filename)
    file=f.read()
    f.close()
    manage_addPageTemplate(obj, id, title=title, text=file)
    return getattr(obj,id)

def addMethod(ob, id, title, file, pref_type=None):
    """ addMethod(ob, id, title, file, pref_type=None):
        add a method to object 'ob', using id 'id' and title 'title'
        if pref_type is a list, try all extensions in pref_type and
        use the first matching file. if pref_type is a string, add
        the file with extentions 'pref_type'. if pref_type is ommited or
        None, try default_extensions. 'file' is a path relative to this
        modules homedir. """
    import os.path
    filename, file_type = getMatchingSourceFile(file, pref_type)
    assert os.path.exists(filename)
    if file_type in PythonScriptFileTypes:
        return addPythonScript(ob, id, filename)
    elif file_type in PageTemplateFileTypes:
        return addPageTemplate(ob, id, title, filename)
    elif file_type in DTMLFileTypes:
        return addDTML(ob, id, title, filename)
    else:
        assert 0

def addImage(obj,id,file):
    filename = normpath(file_path+'/'+file)
    f=open(filename,'rb')

    contents=f.read()
    f.close()
    title=''
    new_id = obj.manage_addImage(id,contents,title=title)
    return new_id
    
def addFile(obj,id,file):
    filename = normpath(file_path+'/'+file)
    f=open(filename,'rb')

    contents=f.read()
    f.close()
    title=''
    new_id = obj.manage_addFile(id,contents,title=title)
    return new_id

def initialize_permissions(ob, allowed_roles):
    if allowed_roles is None:
        return 
    inherit_permissions = ListType is type(allowed_roles)
    ob.manage_permission(View, allowed_roles, inherit_permissions)

